/*
 * Musiccpp
 * A library for MusicShield
 *
 * Copyright (c) 2015 waveshare.net
 * Website    : waveshare.net
 * Author      :still falling leaves
 * Create Time: May 2015
 * Change Log :
 *
 * The MIT License (MIT)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

// #include <SD.h>
#include <MusicPlayer.h>
#include  <avr/pgmspace.h>


// change this to match your SD shield or module;
// Arduino Ethernet shield: pin 4
// Adafruit SD shields and modules: pin 10
// Sparkfun SD shield: pin 8

static unsigned char fileBuf[32];
songtype list[15];            //song list
int ledcount;
key keys[5];

// int inst = 1;

VS10XX VS1053;
playingstatetype playingState;
ctrlStatetype ctrlState;

/* PCM file Header */
unsigned char pcmHeader[44] = {
  'R', 'I', 'F', 'F',
  0xFF, 0xFF, 0xFF, 0xFF,
  'W', 'A', 'V', 'E',
  'f', 'm', 't', ' ',
  0x10, 0, 0, 0,          /* 16 */
  0x1, 0,                 /* PCM */
  0x1, 0,                 /* chan */
  0x40, 0x1F, 0x0, 0x0,   /* sampleRate */
  0x80, 0x3E, 0x0, 0x0,   /* byteRate */
  2, 0,                   /* blockAlign */
  0x10, 0,                /* bitsPerSample */
  'd', 'a', 't', 'a',
  0xFF, 0xFF, 0xFF, 0xFF
};



/**************************************************************/
#define RESOLUTION 65536    // Timer1 is 16 bit
void MusicPlayer::initTimer1(void)        //initialize Timer1 to 100us overflow
{
  TCCR1A = 0;                 // clear control register A
  TCCR1B = _BV(WGM13);        // set mode as phase and frequency correct pwm, stop the timer

  long cycles;
  long microseconds = 500;   //setup microseconds here
  unsigned char clockSelectBits;
  cycles = (F_CPU / 2000000) * microseconds;                                // the counter runs backwards after TOP, interrupt is at BOTTOM so divide microseconds by 2
  if (cycles < RESOLUTION)              clockSelectBits = _BV(CS10);              // no prescale, full xtal
  else if ((cycles >>= 3) < RESOLUTION) clockSelectBits = _BV(CS11);              // prescale by /8
  else if ((cycles >>= 3) < RESOLUTION) clockSelectBits = _BV(CS11) | _BV(CS10);  // prescale by /64
  else if ((cycles >>= 2) < RESOLUTION) clockSelectBits = _BV(CS12);              // prescale by /256
  else if ((cycles >>= 2) < RESOLUTION) clockSelectBits = _BV(CS12) | _BV(CS10);  // prescale by /1024
  else        cycles = RESOLUTION - 1, clockSelectBits = _BV(CS12) | _BV(CS10);  // request was out of bounds, set as maximum

  ICR1 = cycles;
  TCCR1B &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12));
  TCCR1B |= clockSelectBits;                                          // reset clock select register, and starts the clock

  TIMSK1 = _BV(TOIE1);
  TCNT1 = 0;
  sei();                      //enable global interrupt
}


/**************************************************************/
void MusicPlayer::initIO(void)
{
  // pinMode(SD_CS_PIN, OUTPUT);
  // digitalWrite(SD_CS_PIN, HIGH);
  
  pinMode(VS_XCS, OUTPUT);
  digitalWrite(VS_XCS, HIGH);

  pinMode(LED, OUTPUT);
  digitalWrite(LED,LOW);
  	
  //keys input
  pinMode(KEY_VD, INPUT);
  pinMode(KEY_BK, INPUT);
  pinMode(KEY_PS, INPUT);
  pinMode(KEY_NT, INPUT);
  pinMode(KEY_VU, INPUT);
  //internal pull-up
  digitalWrite(KEY_VD, HIGH);
  digitalWrite(KEY_BK, HIGH);
  digitalWrite(KEY_PS, HIGH);
  digitalWrite(KEY_NT, HIGH);
  digitalWrite(KEY_VU, HIGH);
}


/**********************************Midi Player Part***************************/
void MusicPlayer::beginMidi(void)
{
  //Init VS105B in Midi Format
  Serial.println("Init vs10xx in MIDI format...");
  VS1053.initForMidi();
  Serial.println("done\r\n");

  midiVol = 600;
  VS1053.writeRegister(SPI_VOL, midiVol);
}

void MusicPlayer::midiWriteData(char cmd, char high, char low)
{
  while(!digitalRead(VS_DREQ));
  digitalWrite(VS_XDCS, LOW);
  midiSendByte(cmd);
  if((cmd & 0xF0) <= 0xB0 || (cmd & 0xF0) >= 0xE0)
  {
    midiSendByte(high);
    midiSendByte(low);
  }
  else
  {
    midiSendByte(high);
  }
  digitalWrite(VS_XDCS, HIGH);
}

void MusicPlayer::midiNoteOn(char channel, char note, char rate)
{
  midiWriteData((0x90 | channel), note, rate);
}

void MusicPlayer::midiNoteOff(char channel, char note, char rate)
{
  midiWriteData((0x80 | channel), note, rate);
}

void MusicPlayer::midiSendByte(char data)
{
  SPI.transfer(0x00);
  SPI.transfer(data);
}

void MusicPlayer::midiDemoPlayer(void)
{
  delay(1000);
  midiWriteData(0xB0, 0x07, 170);   //channel volume

  //GM2 Mode
  Serial.print("Fancy Midi Sounds\r\n");
  midiWriteData(0xB0, 0, 0x78);     //Bank select drums
  for(int instrument = 30 ; instrument < 31 ; instrument++)
  {
    Serial.print(" Instrument: ");
    Serial.println(instrument, DEC);

    midiWriteData(0xC0, instrument, 0);    //Set instrument number. 0xC0 is a 1 data byte command

    //Play notes from F#-0 (30) to F#-5 (90):
    for (int note = 27 ; note < 87 ; note++)
    {
      Serial.print("SN:");
      Serial.println(note, DEC);
      //Note on channel 1 (0x90), some note value (note), middle velocity (0x45):
      midiNoteOn(0, note, 127);
      delay(50);

      //Turn off the note with a given off/release velocity
      midiNoteOff(0, note, 127);
      delay(50);
    }

    //delay 100ms between each instruments
    delay(100);
    }
}

void MusicPlayer::ccrbMidiPlayer(void)
{
  delay(1000);
  midiWriteData(0xB0, 0x07, 170);   //channel volume

  //GM2 Mode
  Serial.print("Midi Sounds\r\n");
  midiWriteData(0xB0, 0, 0x78);     //Bank select drums
  for(int instrument = 20 ; instrument < 21 ; instrument++)
  {
    Serial.print(" Instrument: ");
    Serial.println(instrument, DEC);

    midiWriteData(0xC0, instrument, 0);    //Set instrument number. 0xC0 is a 1 data byte command

    //Play notes from F#-0 (30) to F#-5 (90):
    for (int note = 40 ; note < 50 ; note++)
    {
      Serial.print("SN:");
      Serial.println(note, DEC);
      //Note on channel 1 (0x90), some note value (note), middle velocity (0x45):
      midiNoteOn(0, note, 100);
      delay(50);

      //Turn off the note with a given off/release velocity
      midiNoteOff(0, note, 100);
      delay(50);
    }

    //delay 100ms between each instruments
    delay(100);
    }
}

void MusicPlayer::MidiPlayerB(void)
{
  int note = 50;
  Serial.println(note, DEC);
  midiNoteOn(0, note, 100);
  delay(50);
  midiNoteOff(0, note, 100);
  delay(200);

  note = 51;
  Serial.println(note, DEC);
  midiNoteOn(0, note, 100);
  delay(50);
  midiNoteOff(0, note, 100);
  delay(200);
}

void MusicPlayer::RealtimeMidi(void)
{

  midiWriteData(0xB0, 0x07, 120); // Set MIDI channel volume
  midiWriteData(0xB0, 0, 0x00);  // Select bank GM1
  midiWriteData(0xC0, 1, 0); // Select instrument (Piano)
  for (int note = 50 ; note < 100 ; note++){
  midiNoteOn(0, note, 127); // Play note (C4)
  delay (50);
  midiNoteOff(0, note, 127); // Stop playing note
  }

  /*
  // beginInMidiFmt();
  midiWriteData(0xB0, 0x07, 120); // Set MIDI channel volume
  midiWriteData(0xB0, 0, 0x00);  // Select bank GM1
  // for(int instrument = 40 ; instrument < 41 ; instrument++){
    midiWriteData(0xC0, 1, 0); // Select instrument (Piano)
    for (int note = 0 ; note < 100 ; note++){
      midiNoteOn(0, note, 127); // Play note (C4)
      delay(20);
      midiNoteOff(0, note, 127); // Stop playing note
      delay(20);
    }
  //}
  */
}

/*

void MusicPlayer::MidiSoundA(void)
{
  midiWriteData(0xB0, 0x07, 120); // Set MIDI channel volume
  midiWriteData(0xB0, 0, 0x00);  // Select bank GM1
  midiWriteData(0xC0, inst, 0); // Select instrument (Piano)
  midiNoteOn(0, 60, 127); // Play note (C4)
  delay (150);
  midiNoteOff(0, 60, 127); // Stop playing note
}

void MusicPlayer::MidiSoundB(void)
{
  midiWriteData(0xB0, 0x07, 120); // Set MIDI channel volume
  midiWriteData(0xB0, 0, 0x00);  // Select bank GM1
  midiWriteData(0xC0, inst, 0); // Select instrument (Piano)
  midiNoteOn(0, 62, 127); // Play note (C4)
  delay (150);
  midiNoteOff(0, 62, 127); // Stop playing note
}

void MusicPlayer::MidiSoundC(void)
{
  midiWriteData(0xB0, 0x07, 120); // Set MIDI channel volume
  midiWriteData(0xB0, 0, 0x00);  // Select bank GM1
  midiWriteData(0xC0, inst, 0); // Select instrument (Piano)
  midiNoteOn(0, 64, 127); // Play note (C4)
  delay (150);
  midiNoteOff(0, 64, 127); // Stop playing note
}

void MusicPlayer::MidiSoundD(void)
{
  midiWriteData(0xB0, 0x07, 120); // Set MIDI channel volume
  midiWriteData(0xB0, 0, 0x00);  // Select bank GM1
  midiWriteData(0xC0, inst, 0); // Select instrument (Piano)
  midiNoteOn(0, 66, 127); // Play note (C4)
  delay (150);
  midiNoteOff(0, 66, 127); // Stop playing note
}

void MusicPlayer::MidiSoundE(void)
{
  midiWriteData(0xB0, 0x07, 120); // Set MIDI channel volume
  midiWriteData(0xB0, 0, 0x00);  // Select bank GM1
  midiWriteData(0xC0, inst, 0); // Select instrument (Piano)
  midiNoteOn(0, 68, 127); // Play note (C4)
  delay (150);
  midiNoteOff(0, 68, 127); // Stop playing note
}

void MusicPlayer::MidiSoundF(void)
{
  midiWriteData(0xB0, 0x07, 120); // Set MIDI channel volume
  midiWriteData(0xB0, 0, 0x00);  // Select bank GM1
  midiWriteData(0xC0, inst, 0); // Select instrument (Piano)
  midiNoteOn(0, 70, 127); // Play note (C4)
  delay (150);
  midiNoteOff(0, 70, 127); // Stop playing note
}

*/

/*
void MusicPlayer::ccrbMidiPlayerB(void)
{
  delay(1000);
  midiWriteData(0xB0, 0x07, 170);   //channel volume

  //GM2 Mode
  Serial.print("Fancy Midi Sounds\r\n");
  midiWriteData(0xB0, 0, 0x78);     //Bank select drums
  for(int instrument = 30 ; instrument < 31 ; instrument++)
  {
    Serial.print(" Instrument: ");
    Serial.println(instrument, DEC);

    midiWriteData(0xC0, instrument, 0);    //Set instrument number. 0xC0 is a 1 data byte command

    int note = 50;
    
    //Play notes from F#-0 (30) to F#-5 (90):
      Serial.print("SN:");
      Serial.println(note, DEC);
      //Note on channel 1 (0x90), some note value (note), middle velocity (0x45):
      midiNoteOn(0, note, 127);
      delay(50);

      //Turn off the note with a given off/release velocity
      midiNoteOff(0, note, 127);
      delay(50);

    //delay 100ms between each instruments
    delay(10);
    }
}*/